# Copyright (c) 2008 Oracle Corporation.  All rights reserved.
#
#Redistribution and use in source and binary forms, with or without
#modification, are permitted provided that the following conditions
#are met:
#
#     - Redistributions of source code must retain the above
#       copyright notice, this list of conditions and the following
#       disclaimer.
#     - Redistributions in binary form must reproduce the above
#       copyright notice, this list of conditions and the following
#       disclaimer in the documentation and/or other materials
#       provided with the distribution.
#     - Neither the name of Oracle Corporation nor the names of its
#       contributors may be used to endorse or promote products
#       derived from this software without specific prior written
#       permission.
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
#COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
#INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
#CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
#LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
#ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#POSSIBILITY OF SUCH DAMAGE.
#
#Alternatively, the contents of this file may be used under the terms
#of the GNU General Public License version 2 (the "GPL") distributed
#with this softwarere in the file COPYING.GPL, in which case the
#provisions of the GPL are applicable instead of the above.
#
#If you wish to allow the use of your version of this file only under
#the terms of the GPL and not to allow others to use your version of
#this file under the license above, indicate your decision by deleting
#the provisions above and replace them with the notice and other
#provisions required by the GPL.  If you do not delete the provisions
#above, a recipient may use your version of this file under the above
#license or the GPL.

import logging, paramiko, os

'''The SCPTransmit class is a simple class with only one object, upload.
upload is a simple function that takes a datastructure assembled
by the p2v-utils using either the text based user interface or a
kickstart configuration file.  The pexpect module is then used to
copy all the files listed to the given server.
'''
class SCPTransmit:

	'''The optional parameters are there to allow the caller to hook the SCPTransmit instance
	to a snack Scale object so that it may update the status as files are copied.
	
	theForm		-
	ui		- SnackScreen object that is hosting the current window
	progress	- The Scale object that will be update via its set() method
	current		- The current value of the Scale
	status		- A TextBox that will display an message
	'''
	def __init__(self, filecopyscreen):
		self.filecopyscreen = filecopyscreen

	'''Basic basic checking of the data.  Raises a basic
	Exception object if there is any error.
	'''
	def __vet(self, p2vdata):
		if not p2vdata:
			self.logger.error('No data was provided to the SCPTransmit module')
			raise Exception('No data was provided to the SCPTransmit module')
		elif not p2vdata.target or p2vdata.target == '':
			self.logger.error('No hostname was provided to the SCPTransmit module')
			raise Exception('No hostname was provided to the SCPTransmit module')
		elif not p2vdata.user or p2vdata.user == '':
			self.logger.error('No user was provided to the SCPTransmit module')
			raise Exception('No user provided')
		elif not p2vdata.passwd or p2vdata.passwd == '':
			self.logger.error('No password was provided to the SCPTransmit module')
			raise Exception('No password provided')
		elif not p2vdata.dir or p2vdata.dir == '':
			self.logger.error('No target directory was provided to the SCPTransmit module')
			raise Exception('No targetdirectory was provided to the SCPTransmit module')
		elif not p2vdata.diskimages or p2vdata.diskimages == []:
			self.logger.error('No list of image files was provided to the SCPTransmit module')
			raise Exception('No list of image files was provided to the SCPTransmit module')

	def __copyfile(self, sftp, localpath, remotepath):
		# I would have used os.path.getsize to determine the size of the file
		# but for whatever reason that returns 0 for block devices.  So instead
		# I open and seek to the end and then call f.tell()
		f = open(localpath, 'r')
		f.seek(0,2)
		size = f.tell()
		f.close()

		self.filecopyscreen.nextFile(os.path.basename(remotepath), size)
		self.filecopyscreen.refresh()

		localfile = open(localpath, 'r')
		remotefile = sftp.open(remotepath, 'w')
		remotefile.set_pipelined(True)

		buf = localfile.read(32768)
		while buf:
			remotefile.write(buf)
			self.filecopyscreen.bytescopied(len(buf))
			self.filecopyscreen.refresh()
			buf = localfile.read(32768)

		self.filecopyscreen.filecopied()
		self.filecopyscreen.refresh()

		localfile.close()
		remotefile.close()
		
	'''The upload function copies files to a server via scp using
	information in the p2vdata data structure.  The expected data
	structure members are:

	getfiles	- A method that returns a list of the full paths to the files to copy
	dir		- the location within the scp server filesystem to place the files
	target		- hostname or IP address of the scp server
	user		- user name to use to login to the scp server
	passwd		- password to use to login to the scp server 

	Any error will result in an exception that will be raised to the caller.

	The upload function takes 3 parameters.  Only the p2vdata
	parameter is required.  Others are primarily for testing
	purposes.

	p2vdata		- the datastructure with the required information
	logger		- a object implementing the python logging interface
			that will be used to log messages
	child		- a object created using the pexpect.spawn function
	'''
	def upload(self, p2vdata, logger = None, child = None):
		if logger:
			self.logger = logger
		else:
			self.logger = logging.getLogger("p2v")

		self.__vet(p2vdata)

		try:
			t = paramiko.Transport((p2vdata.target, 22))
			t.connect(username=p2vdata.user, password=p2vdata.passwd)
		except paramiko.SSHException, e:
			logger.error("The SSH2 negotiation failed, the key provided by the server was invalid, or wrong username/password")
			logger.error(e)
			if t:
				t.close()
			raise e

		try:
			sftp = paramiko.SFTPClient.from_transport(t)
		except Exception, e:
			logger.error("Error starting SFTP client from transport")
			logger.error(e)
			t.close()
			if sftp:
				sftp.close()
			raise e

		try:
			sftp.chdir(p2vdata.dir)
		except Exception, e:
			try:
				sftp.mkdir(p2vdata.dir)
				sftp.chdir(p2vdata.dir)
			except Exception, e:
				logger.error("Error creating target directory %s" %(p2vdata.dir))
				logger.error(e)
				t.close()
				sftp.close()
				raise e
			
		try:
			self.__copyfile(sftp, "/tmp/p2v/vm.cfg", "vm.cfg")
		except Exception, e:
			logger.error("Error copying vm.cfg file")
			logger.error(e)
			t.close()
			sftp.close()
			raise e
			
		try:
			self.__copyfile(sftp, "/tmp/p2v/catalog", "catalog")
		except Exception, e:
			logger.error("Error copying catalog file")
			logger.error(e)
			t.close()
			sftp.close()
			raise e

		for disk in p2vdata.diskimages:
			try:
				self.__copyfile(sftp, disk.path, "System-%s.img" %(disk.device))
			except Exception, e:
				logger.error("Error copying image file %s" %(disk.device))
				logger.error(e)
				t.close()
				sftp.close()
				raise e


		sftp.close()
		t.close()

		return None
