# Copyright (c) 2008 Oracle Corporation.  All rights reserved.
#
#Redistribution and use in source and binary forms, with or without
#modification, are permitted provided that the following conditions
#are met:
#
#     - Redistributions of source code must retain the above
#       copyright notice, this list of conditions and the following
#       disclaimer.
#     - Redistributions in binary form must reproduce the above
#       copyright notice, this list of conditions and the following
#       disclaimer in the documentation and/or other materials
#       provided with the distribution.
#     - Neither the name of Oracle Corporation nor the names of its
#       contributors may be used to endorse or promote products
#       derived from this software without specific prior written
#       permission.
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
#COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
#INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
#CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
#LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
#ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#POSSIBILITY OF SUCH DAMAGE.
#
#Alternatively, the contents of this file may be used under the terms
#of the GNU General Public License version 2 (the "GPL") distributed
#with this softwarere in the file COPYING.GPL, in which case the
#provisions of the GPL are applicable instead of the above.
#
#If you wish to allow the use of your version of this file only under
#the terms of the GPL and not to allow others to use your version of
#this file under the license above, indicate your decision by deleting
#the provisions above and replace them with the notice and other
#provisions required by the GPL.  If you do not delete the provisions
#above, a recipient may use your version of this file under the above
#license or the GPL.

from snack import *
from p2vtui.Step import Step
from p2vtui.common import checkField
from pykickstart.data import KickstartNetworkData
import kudzu, os, isys, socket, fcntl, struct

class NetworkConfig(Step):

	def __init__(self, previous = None, next = "SETUP_DISKS", test = False):
		Step.__init__(self, previous, next)
		self.staticFields = []
		self.alterNetwork =  not test

	# fancy smancy way of getting an IP address for a particlar NIC such as 'eth0'
	def get_ip_address(self, ifname):
		s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
		return socket.inet_ntoa(fcntl.ioctl(
			s.fileno(),
			0x8915,  # SIOCGIFADDR
			struct.pack('256s', ifname[:15])
			)[20:24])

	# sanity check an IP string.
	def checkIP(self, ip_string):
		try:
			socket.inet_pton(socket.AF_INET, ip_string)
			return True
		except socket.error:
			return False 

	# Disables or enables the fields in self.staticfields depending on whether
	# DHCP is choosen
	def DHCPtoggled(self):
		active = self.rb.getSelection()
		if active == "DHCP":
			for field in self.staticFields:
				field.setFlags(FLAG_DISABLED, FLAGS_SET)
		else:
			for field in self.staticFields:
				field.setFlags(FLAG_DISABLED, FLAGS_RESET)

	# Uses kudzu to probe network devices and puts them in a listbox so that
	# one can be choosen
	def fillListBox(self):
		# kudzu.probe returns a list of network device objects each having
		# the following attributes
		# obj[0] = device name (eth0)
		# obj[1] = device driver (e1000)
		# obj[2] = device description
		probedevs = kudzu.probe(kudzu.CLASS_NETWORK, kudzu.BUS_UNSPEC, 0)

		# populate the listbox
		for device in probedevs:
			if device[2] == None:
				device[2] = ""
			self.lb.append("%s -- %s" %(device[0], device[2]), device)

	def storeEntries(self, data):
		net =  KickstartNetworkData()

		usedhcp = self.rb.getSelection()
		if usedhcp == "STATIC":
			net.bootProto = "static"
		else:
			net.bootProto = "dhcp"
		net.device = self.lb.current()
		net.gateway = self.gwentry.value()
		net.ip = self.ipentry.value()
		net.nameserver = self.nsentry.value()
		net.netmask = self.nmentry.value()

		data.network = [ net ]
		data.domain = self.domentry.value()

	def validateData(self, data):
		if len(data.network) > 0:
			return True
		else:
			return False

	# Maybe a little obfuscated but it is really straightfoward.
	# A return value of True means everything checks out, False means there was a problem.
	def inputsOK(self, ui):
		usedhcp = self.rb.getSelection()

		if usedhcp == "STATIC":
			ipmesg = "The IP address should be 4 numbers between 0 and 255 separated by \'.\'"
			nmmesg = "The Netmask should be 4 numbers between 0 and 255 separated by \'.\'"
			gwmesg = "The Gateway should be a valid IP address with 4 numbers between 0 and 255 separated by \'.\'"
			nsmesg = "The Nameserver should be a valid IP address with 4 numbers between 0 and 255 separated by \'.\'"

			# If the return of any checkField is False, we want to continue looping on the network config ui
			# otherwise they are all true and we can move on so we can logically 'and' them together
			return (checkField(ui, self.ipentry.value(), "IP Address", ipmesg, checkfunc = self.checkIP, optional = False) and
				checkField(ui, self.nmentry.value(), "Netmask", nmmesg, checkfunc = self.checkIP, optional = False) and
				checkField(ui, self.gwentry.value(), "Gateway", gwmesg, checkfunc = self.checkIP, optional = False) and
				checkField(ui, self.domentry.value(), "Domain", None, checkfunc = None, optional = True) and
				checkField(ui, self.nsentry.value(), "Nameserver", nsmesg, checkfunc = self.checkIP, optional = True))
		else:
			# no need to check any user input for DHCP case
			return True

	def setupNetworking(self, data):
		# the settings should be stored in the first element of data.network
		net = data.network[0]

		# The kickstart parser might store the device as a simple string rather
		# than an kudzu probe object
		if isinstance(net.device, str):
			device = net.device
		else:
			device = net.device[0]


		# do lo first
		if os.access("/usr/sbin/ifconfig", os.X_OK):
			ifconfigbin = "/usr/sbin/ifconfig"
		elif os.access("/sbin/ifconfig", os.X_OK):
			ifconfigbin = "/sbin/ifconfig"
		else:
			ifconfigbin = "ifconfig"

		try:
			os.system("%s lo 127.0.0.1" %(ifconfigbin))
		except:
			self.logger.error("Error trying to start lo")

		# start up dhcp interfaces first
		if net.bootProto == "dhcp":
			self.logger.info('Attempting to obtain DHCP IP address for %s' %(device))
			try:
				# The isys interface for setting up a NIC with a DHCP address changed
				# slightly from one version of anaconda to the next
				try:
					ns = isys.pumpNetDevice(device)
				except Exception:
					ns = isys.dhcpNetDevice(device)
				if ns:
					self.logger.info('Writing resolv.conf')
					f = open("/etc/resolv.conf", "w")
					f.write("nameserver %s\n" % ns)
					f.close()
				else:
					self.logger.info('Writing resolv.conf')
					f = open("/etc/resolv.conf", "w")
					f.write("search localdomain\n")
					f.write("nameserver 127.0.0.1\n")
					f.close()
			except Exception, e:
				self.logger.error("Error trying to start DHCP on %s in startNetworking()" %(device))
				return e

		else: 
			try:
				isys.configNetDevice(device,
					net.ip,
					net.netmask,
					net.gateway)

				if data.domain or net.nameserver:
					self.logger.info('Writing resolv.conf')
					f = open("/etc/resolv.conf", "w")
					if data.domain:
						f.write("search %s\n" %(data.domain))
					if net.nameserver:
						f.write("nameserver %s\n" %(net.nameserver))
					f.close()
			except Exception, e:
				self.logger.error("Error trying to start %s in startNetworking()" %(device))
				return e

		# store IP address in data object as it will be displayed to user when HTTP is running
		try:
			data.ipaddr = self.get_ip_address(device)
			self.logger.info("IP address %s will be used" %(data.ipaddr))
			return None
		except Exception, e:
			self.logger.info("Error getting IP address for %s" %(device))
			data.ipaddr = "0.0.0.0"
			self.logger.info("IP address %s will be used" %(data.ipaddr))
			return e	

	def createSubGrid(self, data):
		theGrid = Grid(2, 5)

		if len(data.network) > 0:
			net = data.network[0]
		else:
			net = KickstartNetworkData()

		theGrid.setField(Textbox(13, 1, "IP Address:"), 0, 0)
		theGrid.setField(Textbox(13, 1, "Netmask:"), 0, 1)
		theGrid.setField(Textbox(13, 1, "Gateway:"), 0, 2)
		theGrid.setField(Textbox(13, 1, "Domain:"), 0, 3)
		theGrid.setField(Textbox(13, 1, "Nameserver:"), 0, 4)

		self.ipentry = Entry(20)
		self.nmentry = Entry(20)
		self.gwentry = Entry(20)
		self.domentry = Entry(20)
		self.nsentry = Entry(20)

		if net.ip:
			self.ipentry.set(net.ip)
		if net.netmask:
			self.nmentry.set(net.netmask)
		if net.gateway:
			self.gwentry.set(net.gateway)
		if net.nameserver:
			self.nsentry.set(net.nameserver)
		if data.domain:
			self.domentry.set(data.domain)

		if net.bootProto == "dhcp":
			self.ipentry.setFlags(FLAG_DISABLED, FLAGS_SET)
			self.nmentry.setFlags(FLAG_DISABLED, FLAGS_SET)
			self.gwentry.setFlags(FLAG_DISABLED, FLAGS_SET)
			self.domentry.setFlags(FLAG_DISABLED, FLAGS_SET)
			self.nsentry.setFlags(FLAG_DISABLED, FLAGS_SET)

		theGrid.setField(self.ipentry, 1, 0)
		theGrid.setField(self.nmentry, 1, 1)
		theGrid.setField(self.gwentry, 1, 2)
		theGrid.setField(self.domentry, 1, 3)
		theGrid.setField(self.nsentry, 1, 4)

		self.staticFields.append(self.ipentry)
		self.staticFields.append(self.nmentry)
		self.staticFields.append(self.gwentry)
		self.staticFields.append(self.domentry)
		self.staticFields.append(self.nsentry)

		return theGrid
		
	def createGrid(self, ui, data):
		toplevel = GridFormHelp (ui, "Please choose a network interface to use for image transfer", "P2V", 1, 5)

		if len(data.network) > 0:
			net = data.network[0]
		else:
			net = KickstartNetworkData()

		self.lb = Listbox(3, scroll = 1, width = 60)
		self.fillListBox()

		self.bb = ButtonBar(ui, (("OK", "OK", "F12"), ("CANCEL", "CANCEL")))

		usedhcp = True
		if net.bootProto == "static":
			usedhcp = False
		
		# List of buttons for the RadioBar
		self.dhcpButton = ("Automatically obtain via DHCP", "DHCP", usedhcp)
		self.staticButton = ("Manually configure", "STATIC", not usedhcp)
		self.rb = RadioBar(ui, [self.dhcpButton, self.staticButton])
		for button in self.rb.group.buttonlist:
			button[0].setCallback(self.DHCPtoggled)

		toplevel.add(self.lb, 0, 0, (0, 0, 0, 1), growx = 1)
		toplevel.add(self.rb, 0, 1, (0, 0, 0, 0), growx = 1)
		toplevel.add(self.createSubGrid(data), 0, 2, (0, 0, 0, 1), growx = 1)
		toplevel.add(self.bb, 0, 3, growx = 1)
		return toplevel

	def doStep(self, ui, data, logger, prompt):
		self.logger = logger

		if not prompt:
			# We are doing a kickstart install so just validate the data and move on
			if self.validateData(data):
				if self.alterNetwork:
					self.setupNetworking(data)
				else:
					self.logger.info("Test flag was set, not altering network config")
				return (self.next, prompt)
			else:
				prompt = True

		while prompt:
			toplevel = self.createGrid(ui, data)
			result = toplevel.run()
			rc = self.bb.buttonPressed(result)
			self.storeEntries(data)
			if rc == "OK" or rc == "F12":
				if self.inputsOK(ui):
					self.logger.info("User inputs seem valid, setting up the network")
					if self.alterNetwork:
						ret = self.setupNetworking(data)
						if ret:
							ButtonChoiceWindow(ui, "Error setting up networking",
								"The following error occured during network setup: %s" %(ret),
								buttons = [ ("OK", "OK", "F12") ])
							continue
					else:
						self.logger.info("Test flag was set, not altering network config")
					ui.popWindow()
					return ("SETUP_DISKS", prompt)
				else:
					continue
			elif rc == "CANCEL" or rc == "ESC":
				self.logger.info("User choose to CANCEL")
				ui.popWindow()
				return ("CANCEL", prompt)

		return (self.next, prompt)
